@@ -113,16 +113,9 @@ type 'a abstract = 'a Ctypes_static.abstract
 include Ctypes_types.TYPE
  with type 'a typ = 'a Ctypes_static.typ
   and type ('a, 's) field := ('a, 's) field
-include Ctypes_types.FUNCTION
 
 (** {3 Operations on types} *)
 
-val ( *:* ) : 't typ -> 'a typ -> ('a, (('s, [`Struct]) structured as 't)) field
-(** @deprecated Add an anonymous field to a structure.  Use {!field} instead. *)
-
-val ( +:+ ) : 't typ -> 'a typ -> ('a, (('s, [`Union]) structured as 't)) field
-(** @deprecated Add an anonymous field to a union.  Use {!field} instead. *)
-
 val sizeof : 'a typ -> int
 (** [sizeof t] computes the size in bytes of the type [t].  The exception
     {!IncompleteType} is raised if [t] is incomplete. *)
@@ -186,14 +179,20 @@ val to_voidp : _ ptr -> unit ptr
 (** Conversion to [void *]. *)
 
 val allocate : ?finalise:('a ptr -> unit) -> 'a typ -> 'a -> 'a ptr
-(** [allocate t v] allocates a fresh value of type [t], initialises it with
-    [v] and returns its address.  The argument [?finalise], if present, will be
-    called just before the memory is freed.  *)
+(** [allocate t v] allocates a fresh value of type [t], initialises it
+    with [v] and returns its address.  The argument [?finalise], if
+    present, will be called just before the memory is freed.  The value
+    will be automatically freed after no references to the pointer
+    remain within the calling OCaml program. *)
 
 val allocate_n : ?finalise:('a ptr -> unit) -> 'a typ -> count:int -> 'a ptr
-(** [allocate_n t ~count:n] allocates a fresh array with element type [t] and
-    length [n], and returns its address.  The argument [?finalise], if present,
-    will be called just before the memory is freed.  *)
+(** [allocate_n t ~count:n] allocates a fresh array with element type
+    [t] and length [n], and returns its address.  The argument
+    [?finalise], if present, will be called just before the memory is
+    freed.  The array will be automatically freed after no references
+    to the pointer remain within the calling OCaml program.  The
+    memory is allocated with libc's [calloc] and is guaranteed to be
+    zero-filled.  *)
 
 val ptr_compare : 'a ptr -> 'a ptr -> int
 (** If [p] and [q] are pointers to elements [i] and [j] of the same array then
@@ -207,6 +206,9 @@ val reference_type : 'a ptr -> 'a typ
 val ptr_of_raw_address : nativeint -> unit ptr
 (** Convert the numeric representation of an address to a pointer *)
 
+val funptr_of_raw_address : nativeint -> (unit -> unit) Ctypes_static.static_funptr
+(** Convert the numeric representation of an address to a function pointer *)
+
 val raw_address_of_ptr : unit ptr -> nativeint
 (** [raw_address_of_ptr p] returns the numeric representation of p.
 
@@ -275,6 +277,30 @@ sig
       element of the list is the result of reading the corresponding element of
       [a]. *)
 
+  val iter : ('a -> unit) -> 'a t -> unit
+  (** [iter f a] is analogous to [Array.iter f a]: it applies [f] in turn to
+      all the elements of [a]. *)
+
+  val map : 'b typ -> ('a -> 'b) -> 'a t -> 'b t
+  (** [map t f a] is analogous to [Array.map f a]: it creates a new array with
+      element type [t] whose elements are obtained by applying [f] to the
+      elements of [a]]. *)
+
+  val mapi : 'b typ -> (int -> 'a -> 'b) -> 'a t -> 'b t
+  (** [mapi] behaves like {!Array.mapi}, except that it also passes the
+      index of each element as the first argument to [f] and the element
+      itself as the second argument. *)
+
+  val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
+  (** [CArray.fold_left (@) x a] computes 
+         [(((x @ a.(0)) @ a.(1)) ...) @ a.(n-1)]
+       where [n] is the length of the array [a]. *)
+
+  val fold_right : ('b -> 'a -> 'a) -> 'b t -> 'a -> 'a
+  (** [CArray.fold_right f a x] computes
+         [a.(0) @ (a.(1) @ ( ... (a.(n-1) @ x) ...))]
+       where [n] is the length of the array [a]. *)
+
   val length : 'a t -> int
   (** Return the number of elements of the given array. *)
 
@@ -291,8 +317,18 @@ sig
       used to initialise every element of the array.  The argument [?finalise],
       if present, will be called just before the memory is freed. *)
 
+  val copy : 'a t -> 'a t
+  (** [copy a] creates a fresh array with the same elements as [a]. *)
+
+  val sub : 'a t -> pos:int -> length:int -> 'a t
+  (** [sub a ~pos ~length] creates a fresh array of length [length] containing
+      the elements [a.(pos)] to [a.(pos + length - 1)] of [a].
+
+      Raise [Invalid_argument "CArray.sub"] if [pos] and [length] do not
+      designate a valid subarray of [a]. *)
+
   val element_type : 'a t -> 'a typ
-(** Retrieve the element type of an array. *)
+  (** Retrieve the element type of an array. *)
 end
 (** Operations on C arrays. *)
 
@@ -332,8 +368,8 @@ val bigarray_of_array : < element: 'a;
                           carray: 'c carray;
                           dims: 'i > bigarray_class ->
     ('a, 'f) Bigarray.kind -> 'c carray -> 'b
-(** [bigarray_of_array c k a] converts the {!CArray.t} value [c] to a bigarray
-    value.  No copy is made; the result occupies the same memory as [c]. *)
+(** [bigarray_of_array c k a] converts the {!CArray.t} value [a] to a bigarray
+    value.  No copy is made; the result occupies the same memory as [a]. *)
 
 (** {3 Struct and union values} *)
 
@@ -381,7 +417,7 @@ val coerce : 'a typ -> 'b typ -> 'a -> 'b
 
     The following coercions are currently supported:
 
-     - All pointer types are intercoercible.
+     - All function and object pointer types are intercoercible.
      - Any type may be coerced to {!void}
      - There is a coercion between a {!view} and another type [t] (in either
        direction) if there is a coercion between the representation type
@@ -412,6 +448,23 @@ val coerce_fn : 'a fn -> 'b fn -> 'a -> 'b
     ctypes may both add new types of coercion and restrict the existing
     coercions. *)
 
+(** {2:roots Registration of OCaml values as roots} *)
+module Root :
+sig
+  val create : 'a -> unit ptr
+  (** [create v] allocates storage for the address of the OCaml value [v],
+      registers the storage as a root, and returns its address. *)
+
+  val get : unit ptr -> 'a
+  (** [get p] retrieves the OCaml value whose address is stored at [p]. *)
+
+  val set : unit ptr -> 'a -> unit
+  (** [set p v] updates the OCaml value stored as a root at [p]. *)
+
+  val release : unit ptr -> unit
+  (** [release p] unregsiters the root [p]. *)
+end
+
 (** {2 Exceptions} *)
 
 exception Unsupported of string
@@ -437,5 +490,6 @@ exception IncompleteType
     cannot be used as argument or return types.
 *)
 
-exception Uncoercible
+type uncoercible_info
+exception Uncoercible of uncoercible_info
 (** An attempt was made to coerce between uncoercible types.  *)
